#include "stdafx.h"
#include "OpenCVD2D.h"
#include "Helper.h"

// RXgN^
OpenCVD2D::OpenCVD2D(HRESULT* pHr) :
	m_pD2DFactory(NULL),
	m_pHwndRenderTarget(NULL),
	m_pD2DBitmap(NULL)
{
	HRESULT hr = S_OK;

	::HeapSetInformation(NULL, HeapEnableTerminationOnCorruption, NULL, 0);

	// COM̏
	hr = ::CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	if (SUCCEEDED(hr))
	{
		// Direct2D t@Ng̍쐬
		hr = ::D2D1CreateFactory(D2D1_FACTORY_TYPE_SINGLE_THREADED, IID_ID2D1Factory, (void**)&m_pD2DFactory);
	}

	*pHr = hr;
}

// fXgN^
OpenCVD2D::~OpenCVD2D()
{
	SafeRelease(&m_pD2DFactory);
	::CoUninitialize();
}

// foCXˑ\[X̐
HRESULT OpenCVD2D::CreateDeviceResources(HWND hWnd)
{
	HRESULT hr = S_OK;

	// t@NgΉȂ
	if (m_pD2DFactory == NULL)
	{
		return E_FAIL;
	}

	// _[^[Qbg̍쐬
	if (m_pHwndRenderTarget == NULL)
	{
		RECT rect;
		::GetClientRect(hWnd, &rect);
		D2D1_SIZE_U size = D2D1::Size<UINT>(rect.right, rect.bottom);
		hr = m_pD2DFactory->CreateHwndRenderTarget(
			D2D1::RenderTargetProperties(),
			D2D1::HwndRenderTargetProperties(hWnd, size),
			&m_pHwndRenderTarget);
	}

	return hr;
}

// foCXˑ\[X̉
void OpenCVD2D::DeleteDeviceResources()
{
	SafeRelease(&m_pD2DBitmap);
	SafeRelease(&m_pHwndRenderTarget);
}

// 摜ǂݍ
HRESULT OpenCVD2D::load(LPWSTR path)
{
	HRESULT hr = S_OK;

	std::wstring wstrFilename = path;
	// ϊ
	std::string strFilename;
	wStrToMbStr(wstrFilename, strFilename);
	//
	cv::Mat mat = cv::imread(strFilename);
	if (mat.data == NULL)
	{
		hr = E_FAIL;
	}
	else
	{
		m_mat = mat;
		hr = CreateD2DBitmapFromMat();
	}

	return hr;
}

// MatD2DBitmap쐬
HRESULT OpenCVD2D::CreateD2DBitmapFromMat()
{
	if (m_mat.data == NULL)
	{
		return E_FAIL;
	}

	if (m_pHwndRenderTarget == NULL)
	{
		return E_FAIL;
	}

	IWICImagingFactory* pIWICFactory = NULL;

	HRESULT hr = CoCreateInstance(
		CLSID_WICImagingFactory,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_PPV_ARGS(&pIWICFactory)
		);

	BYTE* data = m_mat.ptr();
	UINT stride = m_mat.step1();
	UINT width = m_mat.cols;
	UINT height = m_mat.rows;
	UINT size = stride * height;

	IWICBitmap* pWICBitmap = NULL;

	if (SUCCEEDED(hr))
	{
		hr = pIWICFactory->CreateBitmapFromMemory(
			width,
			height,
			GUID_WICPixelFormat24bppBGR,
			stride,
			size,
			data,
			&pWICBitmap
			);
	}

	IWICFormatConverter *pConverter = NULL;
	if (SUCCEEDED(hr))
	{
		hr = pIWICFactory->CreateFormatConverter(&pConverter);
	}

	if (SUCCEEDED(hr))
	{
		hr = pConverter->Initialize(
			pWICBitmap,
			GUID_WICPixelFormat32bppBGR,    // or GUID_WICPixelFormat32bppPBGRA
			WICBitmapDitherTypeNone,
			NULL,
			0.0f,
			WICBitmapPaletteTypeMedianCut
			);
	}

	if (m_pD2DBitmap != NULL)
	{
		SafeRelease(&m_pD2DBitmap);
		m_pD2DBitmap = NULL;
	}

	if (SUCCEEDED(hr))
	{
		hr = m_pHwndRenderTarget->CreateBitmapFromWicBitmap(
			pConverter,
			NULL,
			&m_pD2DBitmap
			);
	}

	SafeRelease(&pConverter);
	SafeRelease(&pWICBitmap);
	SafeRelease(&pIWICFactory);

	// D2DBitmapɕϊMat̓[XĂOK
	// A摜sꍇ̓[Xi摜Ă܂j
	m_mat.release();

	return hr;
}

// rectɓڂ悤ɉ摜`
void OpenCVD2D::Render(RECT rect)
{
	if (m_pHwndRenderTarget == NULL)
	{
		// `悵Ȃ
		return;
	}

	if (m_pHwndRenderTarget->CheckWindowState() &
		D2D1_WINDOW_STATE_OCCLUDED)
	{
		// `悵Ȃ
		return;
	}

	// _[^[Qbg̘_TCY̍Đݒ
	m_pHwndRenderTarget->Resize(D2D1::SizeU(rect.right,
		rect.bottom));

	m_pHwndRenderTarget->BeginDraw();

	m_pHwndRenderTarget->SetTransform(D2D1::Matrix3x2F::Identity());

	//wiNA
	m_pHwndRenderTarget->Clear(
		D2D1::ColorF(0.8f, 0.8f, 0.8f));

	if (m_pD2DBitmap != NULL)
	{
		D2D1_SIZE_F rtSize = m_pHwndRenderTarget->GetSize();
		// Z^[킹
		float ratio = 0.0;
		D2D1_RECT_F rectangle;
		D2D1_SIZE_U imageSize = m_pD2DBitmap->GetPixelSize();
		float imageAspect = (float)imageSize.width / (float)imageSize.height;
		float rtAspect = (float)rtSize.width / (float)rtSize.height;
		if (rtAspect > imageAspect)
		{
			// ʂ̕
			// cɍ킹
			ratio = imageSize.height / rtSize.height;
			float imageWidth = (float)imageSize.width / ratio;
			float pad = (rtSize.width - imageWidth) / 2.0f;
			rectangle = D2D1::RectF(pad, 0.0f, pad + imageWidth, rtSize.height);
		}
		else
		{
			// ʂ̕c
			// ɍ킹
			ratio = imageSize.width / rtSize.width;
			float imageHeight = (float)imageSize.height / ratio;
			float pad = (rtSize.height - imageHeight) / 2.0f;
			rectangle = D2D1::RectF(0.0f, pad, rtSize.width, pad + imageHeight);
		}

		// 摜`
		m_pHwndRenderTarget->DrawBitmap(m_pD2DBitmap, rectangle);
	}

	m_pHwndRenderTarget->EndDraw();
}
